export class Main extends Indicator {

    constructor() {
        super();

        this.setProperty(Property.SeparateWindow, true);

        this.addInput("ADXPeriod", InputType.int, 14);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffers = {
            ADXBuffer: this.addBuffer(),
            PlusDiBuffer: this.addBuffer(),
            MinusDiBuffer: this.addBuffer()
        };

        this.arr = {
            PlusSdiBuffer: [],
            MinusSdiBuffer: [],
            TempBuffer: []
        };
    }

    onInit() {
        let {ADXPeriod} = this.getInputs(),
            shortName = "ADX(" + ADXPeriod + ")";

        this.buffers.ADXBuffer
            .setShape(Shape.Line)
            .setDrawingStyle(DrawingStyle.Solid)
            .setLabel("ADX")
            .setColor(Color.LightSeaGreen)
            .setShortName(shortName)
            .setDrawBegin(ADXPeriod);

        this.buffers.PlusDiBuffer
            .setShape(Shape.Line)
            .setDrawingStyle(DrawingStyle.Solid)
            .setLabel("+DI")
            .setColor(Color.YellowGreen)
            .setShortName(shortName)
            .setDrawBegin(ADXPeriod);

        this.buffers.MinusDiBuffer
            .setShape(Shape.Line)
            .setDrawingStyle(DrawingStyle.Solid)
            .setLabel("-DI")
            .setColor(Color.Wheat)
            .setShortName(shortName)
            .setDrawBegin(ADXPeriod);
    }

    async onUpdate() {
        const {
                High,
                Low,
                Close,
            } = await Bar.load([
                Bar.Mode.High,
                Bar.Mode.Low,
                Bar.Mode.Close,
            ], PriceType),
            barSize = Close.length,
            {ADXPeriod, PriceType} = this.getInputs();
        let pdm, mdm, tr, price_high, price_low, starti, i, counted_bars = Bar.counted(),
            num1, num2, num3, list, nextClose;

        i = barSize - 2;
        this.arr.PlusSdiBuffer[i + 1] = 0;
        this.arr.MinusSdiBuffer[i + 1] = 0;
        if (counted_bars >= i) i = barSize - counted_bars - 1;
        starti = i;

        while (i >= 0) {
            price_low = Low[i];
            price_high = High[i];

            pdm = price_high - High[i + 1];
            mdm = Low[i + 1] - price_low;
            nextClose = Close[i + 1] || 0;
            if (pdm < 0) {
                pdm = 0;
            }
            if (mdm < 0) {
                mdm = 0;
            }
            if (pdm === mdm) {
                pdm = 0;
                mdm = 0;
            }
            else if (pdm < mdm) {
                pdm = 0;
            }
            else if (mdm < pdm) {
                mdm = 0;
            }
            num1 = Math.abs(price_high - price_low);
            num2 = Math.abs(price_high - nextClose);
            num3 = Math.abs(price_low - nextClose);
            tr = Math.max(num1, num2);
            tr = Math.max(tr, num3);
            if (tr === 0) {
                this.arr.PlusSdiBuffer[i] = 0;
                this.arr.MinusSdiBuffer[i] = 0;
            }
            else {
                this.arr.PlusSdiBuffer[i] = 100.0 * pdm / tr;
                this.arr.MinusSdiBuffer[i] = 100.0 * mdm / tr;
            }
            i--;
        }
        if (counted_bars > 0) counted_bars--;
        let limit = barSize - counted_bars;
        list = Indicators.MAOnArray(this.arr.PlusSdiBuffer, barSize, ADXPeriod, 0, Averaging.Exponential);
        for (i = 0; i <= limit; i++) {
            this.buffers.PlusDiBuffer.set(i, list[i]);
        }
        list = Indicators.MAOnArray(this.arr.MinusSdiBuffer, barSize, ADXPeriod, 0, Averaging.Exponential);
        for (i = 0; i <= limit; i++) {
            this.buffers.MinusDiBuffer.set(i, list[i]);
        }
        i = barSize - 2;
        this.arr.TempBuffer[i + 1] = 0;
        i = starti;
        while (i >= 0) {
            let div = Math.abs(this.buffers.PlusDiBuffer.get(i) + this.buffers.MinusDiBuffer.get(i));
            if (div === 0) {
                this.arr.TempBuffer[i] = 0;
            }
            else {
                this.arr.TempBuffer[i] = 100 * (Math.abs(this.buffers.PlusDiBuffer.get(i) - this.buffers.MinusDiBuffer.get(i)) / div);
            }
            i--;
        }
        list = Indicators.MAOnArray(this.arr.TempBuffer, barSize, ADXPeriod, 0, Averaging.Exponential);
        for (i = 0; i < limit; i++) {
            this.buffers.ADXBuffer.set(i, list[i]);
        }
    }
}